package java.net;

import java.nio.charset.CoderResult;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CodingErrorAction;
import java.text.Normalizer;
import java.nio.ByteBuffer;
import java.nio.charset.CharacterCodingException;
import java.nio.CharBuffer;
import sun.nio.cs.ThreadLocalCoders;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;

public final class URI implements Comparable<URI>, Serializable
{
    static final long serialVersionUID = -6052424284110960213L;
    private transient String scheme;
    private transient String fragment;
    private transient String authority;
    private transient String userInfo;
    private transient String host;
    private transient int port;
    private transient String path;
    private transient String query;
    private transient volatile String schemeSpecificPart;
    private transient volatile int hash;
    private transient volatile String decodedUserInfo;
    private transient volatile String decodedAuthority;
    private transient volatile String decodedPath;
    private transient volatile String decodedQuery;
    private transient volatile String decodedFragment;
    private transient volatile String decodedSchemeSpecificPart;
    private volatile String string;
    private static final long L_DIGIT;
    private static final long H_DIGIT = 0L;
    private static final long L_UPALPHA = 0L;
    private static final long H_UPALPHA;
    private static final long L_LOWALPHA = 0L;
    private static final long H_LOWALPHA;
    private static final long L_ALPHA = 0L;
    private static final long H_ALPHA;
    private static final long L_ALPHANUM;
    private static final long H_ALPHANUM;
    private static final long L_HEX;
    private static final long H_HEX;
    private static final long L_MARK;
    private static final long H_MARK;
    private static final long L_UNRESERVED;
    private static final long H_UNRESERVED;
    private static final long L_RESERVED;
    private static final long H_RESERVED;
    private static final long L_ESCAPED = 1L;
    private static final long H_ESCAPED = 0L;
    private static final long L_URIC;
    private static final long H_URIC;
    private static final long L_PCHAR;
    private static final long H_PCHAR;
    private static final long L_PATH;
    private static final long H_PATH;
    private static final long L_DASH;
    private static final long H_DASH;
    private static final long L_DOT;
    private static final long H_DOT;
    private static final long L_USERINFO;
    private static final long H_USERINFO;
    private static final long L_REG_NAME;
    private static final long H_REG_NAME;
    private static final long L_SERVER;
    private static final long H_SERVER;
    private static final long L_SERVER_PERCENT;
    private static final long H_SERVER_PERCENT;
    private static final long L_LEFT_BRACKET;
    private static final long H_LEFT_BRACKET;
    private static final long L_SCHEME;
    private static final long H_SCHEME;
    private static final long L_URIC_NO_SLASH;
    private static final long H_URIC_NO_SLASH;
    private static final char[] hexDigits;
    static final /* synthetic */ boolean $assertionsDisabled;
    
    private URI() {
        this.port = -1;
        this.decodedUserInfo = null;
        this.decodedAuthority = null;
        this.decodedPath = null;
        this.decodedQuery = null;
        this.decodedFragment = null;
        this.decodedSchemeSpecificPart = null;
    }
    
    public URI(String s) throws URISyntaxException {
        this.port = -1;
        this.decodedUserInfo = null;
        this.decodedAuthority = null;
        this.decodedPath = null;
        this.decodedQuery = null;
        this.decodedFragment = null;
        this.decodedSchemeSpecificPart = null;
        new URI.Parser(s).parse(false);
    }
    
    public URI(String s, String s2, String s3, int n, String s4, String s5, String s6) throws URISyntaxException {
        this.port = -1;
        this.decodedUserInfo = null;
        this.decodedAuthority = null;
        this.decodedPath = null;
        this.decodedQuery = null;
        this.decodedFragment = null;
        this.decodedSchemeSpecificPart = null;
        String string = this.toString(s, null, null, s2, s3, n, s4, s5, s6);
        checkPath(string, s, s4);
        new URI.Parser(string).parse(true);
    }
    
    public URI(String s, String s2, String s3, String s4, String s5) throws URISyntaxException {
        this.port = -1;
        this.decodedUserInfo = null;
        this.decodedAuthority = null;
        this.decodedPath = null;
        this.decodedQuery = null;
        this.decodedFragment = null;
        this.decodedSchemeSpecificPart = null;
        String string = this.toString(s, null, s2, null, null, -1, s3, s4, s5);
        checkPath(string, s, s3);
        new URI.Parser(string).parse(false);
    }
    
    public URI(String s, String s2, String s3, String s4) throws URISyntaxException {
        this(s, null, s2, -1, s3, null, s4);
    }
    
    public URI(String s, String s2, String s3) throws URISyntaxException {
        this.port = -1;
        this.decodedUserInfo = null;
        this.decodedAuthority = null;
        this.decodedPath = null;
        this.decodedQuery = null;
        this.decodedFragment = null;
        this.decodedSchemeSpecificPart = null;
        new URI.Parser(this.toString(s, s2, null, null, null, -1, null, null, s3)).parse(false);
    }
    
    public static URI create(String s) {
        try {
            return new URI(s);
        }
        catch (URISyntaxException cause) {
            throw new IllegalArgumentException(cause.getMessage(), cause);
        }
    }
    
    public URI parseServerAuthority() throws URISyntaxException {
        if (this.host != null || this.authority == null) {
            return this;
        }
        this.defineString();
        new URI.Parser(this.string).parse(true);
        return this;
    }
    
    public URI normalize() {
        return normalize(this);
    }
    
    public URI resolve(URI uri) {
        return resolve(this, uri);
    }
    
    public URI resolve(String s) {
        return this.resolve(create(s));
    }
    
    public URI relativize(URI uri) {
        return relativize(this, uri);
    }
    
    public URL toURL() throws MalformedURLException {
        if (!this.isAbsolute()) {
            throw new IllegalArgumentException("URI is not absolute");
        }
        return new URL(this.toString());
    }
    
    public String getScheme() {
        return this.scheme;
    }
    
    public boolean isAbsolute() {
        return this.scheme != null;
    }
    
    public boolean isOpaque() {
        return this.path == null;
    }
    
    public String getRawSchemeSpecificPart() {
        this.defineSchemeSpecificPart();
        return this.schemeSpecificPart;
    }
    
    public String getSchemeSpecificPart() {
        if (this.decodedSchemeSpecificPart == null) {
            this.decodedSchemeSpecificPart = decode(this.getRawSchemeSpecificPart());
        }
        return this.decodedSchemeSpecificPart;
    }
    
    public String getRawAuthority() {
        return this.authority;
    }
    
    public String getAuthority() {
        if (this.decodedAuthority == null) {
            this.decodedAuthority = decode(this.authority);
        }
        return this.decodedAuthority;
    }
    
    public String getRawUserInfo() {
        return this.userInfo;
    }
    
    public String getUserInfo() {
        if (this.decodedUserInfo == null && this.userInfo != null) {
            this.decodedUserInfo = decode(this.userInfo);
        }
        return this.decodedUserInfo;
    }
    
    public String getHost() {
        return this.host;
    }
    
    public int getPort() {
        return this.port;
    }
    
    public String getRawPath() {
        return this.path;
    }
    
    public String getPath() {
        if (this.decodedPath == null && this.path != null) {
            this.decodedPath = decode(this.path);
        }
        return this.decodedPath;
    }
    
    public String getRawQuery() {
        return this.query;
    }
    
    public String getQuery() {
        if (this.decodedQuery == null && this.query != null) {
            this.decodedQuery = decode(this.query);
        }
        return this.decodedQuery;
    }
    
    public String getRawFragment() {
        return this.fragment;
    }
    
    public String getFragment() {
        if (this.decodedFragment == null && this.fragment != null) {
            this.decodedFragment = decode(this.fragment);
        }
        return this.decodedFragment;
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof URI)) {
            return false;
        }
        URI uri = (URI)o;
        if (this.isOpaque() != uri.isOpaque()) {
            return false;
        }
        if (!equalIgnoringCase(this.scheme, uri.scheme)) {
            return false;
        }
        if (!equal(this.fragment, uri.fragment)) {
            return false;
        }
        if (this.isOpaque()) {
            return equal(this.schemeSpecificPart, uri.schemeSpecificPart);
        }
        if (!equal(this.path, uri.path)) {
            return false;
        }
        if (!equal(this.query, uri.query)) {
            return false;
        }
        if (this.authority == uri.authority) {
            return true;
        }
        if (this.host != null) {
            if (!equal(this.userInfo, uri.userInfo)) {
                return false;
            }
            if (!equalIgnoringCase(this.host, uri.host)) {
                return false;
            }
            if (this.port != uri.port) {
                return false;
            }
        }
        else if (this.authority != null) {
            if (!equal(this.authority, uri.authority)) {
                return false;
            }
        }
        else if (this.authority != uri.authority) {
            return false;
        }
        return true;
    }
    
    @Override
    public int hashCode() {
        if (this.hash != 0) {
            return this.hash;
        }
        int hash = hash(hashIgnoringCase(0, this.scheme), this.fragment);
        int hash3;
        if (this.isOpaque()) {
            hash3 = hash(hash, this.schemeSpecificPart);
        }
        else {
            int hash2 = hash(hash(hash, this.path), this.query);
            if (this.host != null) {
                hash3 = hashIgnoringCase(hash(hash2, this.userInfo), this.host) + 1949 * this.port;
            }
            else {
                hash3 = hash(hash2, this.authority);
            }
        }
        return this.hash = hash3;
    }
    
    @Override
    public int compareTo(URI uri) {
        int compareIgnoringCase;
        if ((compareIgnoringCase = compareIgnoringCase(this.scheme, uri.scheme)) != 0) {
            return compareIgnoringCase;
        }
        if (this.isOpaque()) {
            if (uri.isOpaque()) {
                int compare5;
                if ((compare5 = compare(this.schemeSpecificPart, uri.schemeSpecificPart)) != 0) {
                    return compare5;
                }
                return compare(this.fragment, uri.fragment);
            }
            else {
                return 1;
            }
        }
        else {
            if (uri.isOpaque()) {
                return -1;
            }
            if (this.host != null && uri.host != null) {
                int compare2;
                if ((compare2 = compare(this.userInfo, uri.userInfo)) != 0) {
                    return compare2;
                }
                int compareIgnoringCase2;
                if ((compareIgnoringCase2 = compareIgnoringCase(this.host, uri.host)) != 0) {
                    return compareIgnoringCase2;
                }
                int n;
                if ((n = this.port - uri.port) != 0) {
                    return n;
                }
            }
            else {
                int compare;
                if ((compare = compare(this.authority, uri.authority)) != 0) {
                    return compare;
                }
            }
            int compare3;
            if ((compare3 = compare(this.path, uri.path)) != 0) {
                return compare3;
            }
            int compare4;
            if ((compare4 = compare(this.query, uri.query)) != 0) {
                return compare4;
            }
            return compare(this.fragment, uri.fragment);
        }
    }
    
    @Override
    public String toString() {
        this.defineString();
        return this.string;
    }
    
    public String toASCIIString() {
        this.defineString();
        return encode(this.string);
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        this.defineString();
        objectOutputStream.defaultWriteObject();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException {
        this.port = -1;
        objectInputStream.defaultReadObject();
        try {
            new URI.Parser(this.string).parse(false);
        }
        catch (URISyntaxException cause) {
            InvalidObjectException ex = new InvalidObjectException("Invalid URI");
            ex.initCause(cause);
            throw ex;
        }
    }
    
    private static int toLower(char c) {
        if (c >= 'A' && c <= 'Z') {
            return c + 32;
        }
        return c;
    }
    
    private static int toUpper(char c) {
        if (c >= 'a' && c <= 'z') {
            return c - 32;
        }
        return c;
    }
    
    private static boolean equal(String s, String anObject) {
        if (s == anObject) {
            return true;
        }
        if (s != null && anObject != null) {
            if (s.length() != anObject.length()) {
                return false;
            }
            if (s.indexOf(37) < 0) {
                return s.equals(anObject);
            }
            int length = s.length();
            int i = 0;
            while (i < length) {
                int char1 = s.charAt(i);
                int char2 = anObject.charAt(i);
                if (char1 != '%') {
                    if (char1 != char2) {
                        return false;
                    }
                    ++i;
                }
                else {
                    if (char2 != '%') {
                        return false;
                    }
                    ++i;
                    if (toLower(s.charAt(i)) != toLower(anObject.charAt(i))) {
                        return false;
                    }
                    ++i;
                    if (toLower(s.charAt(i)) != toLower(anObject.charAt(i))) {
                        return false;
                    }
                    ++i;
                }
            }
            return true;
        }
        else {
            return false;
        }
    }
    
    private static boolean equalIgnoringCase(String s, String s2) {
        if (s == s2) {
            return true;
        }
        if (s != null && s2 != null) {
            int length = s.length();
            if (s2.length() != length) {
                return false;
            }
            int i = 0;
            while (i < length) {
                if (toLower(s.charAt(i)) != toLower(s2.charAt(i))) {
                    return false;
                }
                ++i;
            }
            return true;
        }
        else {
            return false;
        }
    }
    
    private static int hash(int n, String s) {
        if (s == null) {
            return n;
        }
        return (s.indexOf(37) < 0) ? (n * 127 + s.hashCode()) : normalizedHash(n, s);
    }
    
    private static int normalizedHash(int n, String s) {
        int n2 = 0;
        for (int i = 0; i < s.length(); ++i) {
            int char1 = s.charAt(i);
            n2 = 31 * n2 + char1;
            if (char1 == '%') {
                for (int j = i + 1; j < i + 3; ++j) {
                    n2 = 31 * n2 + toUpper(s.charAt(j));
                }
                i += 2;
            }
        }
        return n * 127 + n2;
    }
    
    private static int hashIgnoringCase(int n, String s) {
        if (s == null) {
            return n;
        }
        for (int length = s.length(), i = 0; i < length; ++i) {
            n = 31 * n + toLower(s.charAt(i));
        }
        return n;
    }
    
    private static int compare(String s, String anotherString) {
        if (s == anotherString) {
            return 0;
        }
        if (s != null) {
            if (anotherString != null) {
                return s.compareTo(anotherString);
            }
            return 1;
        }
        else {
            return -1;
        }
    }
    
    private static int compareIgnoringCase(String s, String s2) {
        if (s == s2) {
            return 0;
        }
        if (s != null) {
            if (s2 != null) {
                int length = s.length();
                int length2 = s2.length();
                length2 = ((length < length2) ? length : length2);
                int i = 0;
                while (i < length2) {
                    int n = toLower(s.charAt(i)) - toLower(s2.charAt(i));
                    if (n != 0) {
                        return n;
                    }
                    ++i;
                }
                return length - length2;
            }
            return 1;
        }
        else {
            return -1;
        }
    }
    
    private static void checkPath(String s, String s2, String s3) throws URISyntaxException {
        if (s2 != null && s3 != null && s3.length() > 0 && s3.charAt(0) != '/') {
            throw new URISyntaxException(s, "Relative path in absolute URI");
        }
        return;
    }
    
    private void appendAuthority(StringBuffer sb, String substring, String s, String str, int i) {
        if (str != null) {
            sb.append("//");
            if (s != null) {
                sb.append(quote(s, URI.L_USERINFO, URI.H_USERINFO));
                sb.append('@');
            }
            int n = str.indexOf(58) >= 0 && !str.startsWith("[") && !str.endsWith("]");
            if (!(n == 0)) {
                sb.append('[');
            }
            sb.append(str);
            if (!(n == 0)) {
                sb.append(']');
            }
            if (i != -1) {
                sb.append(':');
                sb.append(i);
            }
        }
        else if (substring != null) {
            sb.append("//");
            if (substring.startsWith("[")) {
                int index = substring.indexOf("]");
                String substring2 = "";
                if (index != -1 && substring.indexOf(":") != -1) {
                    if (index == substring.length()) {
                        substring2 = substring;
                        substring = "";
                    }
                    else {
                        substring2 = substring.substring(0, index + 1);
                        substring = substring.substring(index + 1);
                    }
                }
                sb.append(substring2);
                sb.append(quote(substring, URI.L_REG_NAME | URI.L_SERVER, URI.H_REG_NAME | URI.H_SERVER));
            }
            else {
                sb.append(quote(substring, URI.L_REG_NAME | URI.L_SERVER, URI.H_REG_NAME | URI.H_SERVER));
            }
        }
    }
    
    private void appendSchemeSpecificPart(StringBuffer sb, String substring, String s, String s2, String s3, int n, String s4, String s5) {
        if (substring != null) {
            if (substring.startsWith("//[")) {
                int index = substring.indexOf("]");
                if (index != -1 && substring.indexOf(":") != -1) {
                    String substring2;
                    if (index == substring.length()) {
                        substring2 = "";
                    }
                    else {
                        substring = substring.substring(0, index + 1);
                        substring2 = substring.substring(index + 1);
                    }
                    sb.append(substring);
                    sb.append(quote(substring2, URI.L_URIC, URI.H_URIC));
                }
            }
            else {
                sb.append(quote(substring, URI.L_URIC, URI.H_URIC));
            }
        }
        else {
            this.appendAuthority(sb, s, s2, s3, n);
            if (s4 != null) {
                sb.append(quote(s4, URI.L_PATH, URI.H_PATH));
            }
            if (s5 != null) {
                sb.append('?');
                sb.append(quote(s5, URI.L_URIC, URI.H_URIC));
            }
        }
    }
    
    private void appendFragment(StringBuffer sb, String s) {
        if (s != null) {
            sb.append('#');
            sb.append(quote(s, URI.L_URIC, URI.H_URIC));
        }
    }
    
    private String toString(String str, String s, String s2, String s3, String s4, int n, String s5, String s6, String s7) {
        StringBuffer sb = new StringBuffer();
        if (str != null) {
            sb.append(str);
            sb.append(':');
        }
        this.appendSchemeSpecificPart(sb, s, s2, s3, s4, n, s5, s6);
        this.appendFragment(sb, s7);
        return sb.toString();
    }
    
    private void defineSchemeSpecificPart() {
        if (this.schemeSpecificPart != null) {
            return;
        }
        StringBuffer sb = new StringBuffer();
        this.appendSchemeSpecificPart(sb, null, this.getAuthority(), this.getUserInfo(), this.host, this.port, this.getPath(), this.getQuery());
        this.schemeSpecificPart = sb.toString();
        return;
    }
    
    private void defineString() {
        if (this.string != null) {
            return;
        }
        StringBuffer sb = new StringBuffer();
        if (this.scheme != null) {
            sb.append(this.scheme);
            sb.append(':');
        }
        if (this.isOpaque()) {
            sb.append(this.schemeSpecificPart);
        }
        else {
            if (this.host != null) {
                sb.append("//");
                if (this.userInfo != null) {
                    sb.append(this.userInfo);
                    sb.append('@');
                }
                int n = this.host.indexOf(58) >= 0 && !this.host.startsWith("[") && !this.host.endsWith("]");
                if (!(n == 0)) {
                    sb.append('[');
                }
                sb.append(this.host);
                if (!(n == 0)) {
                    sb.append(']');
                }
                if (this.port != -1) {
                    sb.append(':');
                    sb.append(this.port);
                }
            }
            else if (this.authority != null) {
                sb.append("//");
                sb.append(this.authority);
            }
            if (this.path != null) {
                sb.append(this.path);
            }
            if (this.query != null) {
                sb.append('?');
                sb.append(this.query);
            }
        }
        if (this.fragment != null) {
            sb.append('#');
            sb.append(this.fragment);
        }
        this.string = sb.toString();
        return;
    }
    
    private static String resolvePath(String s, String str, boolean b) {
        int lastIndex = s.lastIndexOf(47);
        int length = str.length();
        String s2 = "";
        if (length == 0) {
            if (lastIndex >= 0) {
                s2 = s.substring(0, lastIndex + 1);
            }
        }
        else {
            StringBuffer sb = new StringBuffer(s.length() + length);
            if (lastIndex >= 0) {
                sb.append(s.substring(0, lastIndex + 1));
            }
            sb.append(str);
            s2 = sb.toString();
        }
        return normalize(s2);
    }
    
    private static URI resolve(URI uri, URI uri2) {
        if (uri2.isOpaque() || uri.isOpaque()) {
            return uri2;
        }
        if (uri2.scheme == null && uri2.authority == null && uri2.path.equals("") && uri2.fragment != null && uri2.query == null) {
            if (uri.fragment != null && uri2.fragment.equals(uri.fragment)) {
                return uri;
            }
            URI uri4 = new URI();
            uri4.scheme = uri.scheme;
            uri4.authority = uri.authority;
            uri4.userInfo = uri.userInfo;
            uri4.host = uri.host;
            uri4.port = uri.port;
            uri4.path = uri.path;
            uri4.fragment = uri2.fragment;
            uri4.query = uri.query;
            return uri4;
        }
        else {
            if (uri2.scheme != null) {
                return uri2;
            }
            URI uri3 = new URI();
            uri3.scheme = uri.scheme;
            uri3.query = uri2.query;
            uri3.fragment = uri2.fragment;
            if (uri2.authority == null) {
                uri3.authority = uri.authority;
                uri3.host = uri.host;
                uri3.userInfo = uri.userInfo;
                uri3.port = uri.port;
                String s = (uri2.path == null) ? "" : uri2.path;
                if (s.length() > 0 && s.charAt(0) == '/') {
                    uri3.path = uri2.path;
                }
                else {
                    uri3.path = resolvePath(uri.path, s, uri.isAbsolute());
                }
                return uri3;
            }
            else {
                uri3.authority = uri2.authority;
                uri3.host = uri2.host;
                uri3.userInfo = uri2.userInfo;
                uri3.host = uri2.host;
                uri3.port = uri2.port;
                uri3.path = uri2.path;
                return uri3;
            }
        }
    }
    
    private static URI normalize(URI uri) {
        if (uri.isOpaque() || uri.path == null || uri.path.length() == 0) {
            return uri;
        }
        String normalize = normalize(uri.path);
        if (normalize == uri.path) {
            return uri;
        }
        URI uri2 = new URI();
        uri2.scheme = uri.scheme;
        uri2.fragment = uri.fragment;
        uri2.authority = uri.authority;
        uri2.userInfo = uri.userInfo;
        uri2.host = uri.host;
        uri2.port = uri.port;
        uri2.path = normalize;
        uri2.query = uri.query;
        return uri2;
    }
    
    private static URI relativize(URI uri, URI uri2) {
        if (uri2.isOpaque() || uri.isOpaque()) {
            return uri2;
        }
        if (!equalIgnoringCase(uri.scheme, uri2.scheme) || !equal(uri.authority, uri2.authority)) {
            return uri2;
        }
        String s = normalize(uri.path);
        String normalize = normalize(uri2.path);
        if (!s.equals(normalize)) {
            if (!s.endsWith("/")) {
                s += "/";
            }
            if (!normalize.startsWith(s)) {
                return uri2;
            }
        }
        URI o = new URI();
        o.path = normalize.substring(s.length());
        o.query = uri2.query;
        o.fragment = uri2.fragment;
        return o;
    }
    
    private static int needsNormalization(String s) {
        boolean n = true;
        int n2 = 0;
        int n3;
        int i;
        for (n3 = s.length() - 1, i = 0; i <= n3 && s.charAt(i) == '/'; ++i) {}
        if (i > 1) {
            n = false;
        }
        while (i <= n3) {
            if (s.charAt(i) == '.' && (i == n3 || s.charAt(i + 1) == '/' || (s.charAt(i + 1) == '.' && (i + 1 == n3 || s.charAt(i + 2) == '/')))) {
                n = false;
            }
            ++n2;
            while (i <= n3) {
                if (s.charAt(i++) != '/') {
                    continue;
                }
                while (i <= n3) {
                    if (s.charAt(i) != '/') {
                        break;
                    }
                    n = false;
                    ++i;
                }
                break;
            }
        }
        return n ? -1 : n2;
    }
    
    private static void split(char[] array, int[] array2) {
        int n = array.length - 1;
        int i = 0;
        int n2 = 0;
        while (i <= n) {
            if (array[i] != '/') {
                break;
            }
            array[i] = '\0';
            ++i;
        }
        while (i <= n) {
            array2[n2++] = i++;
            while (i <= n) {
                if (array[i++] != '/') {
                    continue;
                }
                array[i - 1] = '\0';
                while (i <= n) {
                    if (array[i] != '/') {
                        break;
                    }
                    array[i++] = '\0';
                }
                break;
            }
        }
        if (n2 != array2.length) {
            throw new InternalError();
        }
        return;
    }
    
    private static int join(char[] array, int[] array2) {
        int length = array2.length;
        int n = array.length - 1;
        int n2 = 0;
        if (array[n2] == '\0') {
            array[n2++] = '/';
        }
        for (int n3 : array2) {
            if (n3 != -1) {
                if (n2 == n3) {
                    while (n2 <= n && array[n2] != '\0') {
                        ++n2;
                    }
                    if (n2 <= n) {
                        array[n2++] = '/';
                    }
                }
                else if (n2 < n3) {
                    while (n3 <= n && array[n3] != '\0') {
                        array[n2++] = array[n3++];
                    }
                    if (n3 <= n) {
                        array[n2++] = '/';
                    }
                }
                else {
                    throw new InternalError();
                }
            }
        }
        return n2;
    }
    
    private static void removeDots(char[] array, int[] array2) {
        int length = array2.length;
        int n = array.length - 1;
        int i = 0;
        while (i < length) {
            int n2 = 0;
            do {
                int n3 = array2[i];
                if (array[n3] == '.') {
                    if (n3 == n) {
                        n2 = 1;
                        break;
                    }
                    if (array[n3 + 1] == '\0') {
                        n2 = 1;
                        break;
                    }
                    if (array[n3 + 1] == '.' && (n3 + 1 == n || array[n3 + 2] == '\0')) {
                        n2 = 2;
                        break;
                    }
                    continue;
                }
                else {
                    continue;
                }
            } while (++i < length);
            if (i <= length) {
                if (n2 == 0) {
                    break;
                }
                if (n2 == 1) {
                    array2[i] = -1;
                }
                else {
                    int n4;
                    for (n4 = i - 1; n4 >= 0 && array2[n4] == -1; --n4) {}
                    if (n4 >= 0) {
                        int n5 = array2[n4];
                        if (array[n5] != '.' || array[n5 + 1] != '.' || array[n5 + 2] != '\0') {
                            array2[n4] = (array2[i] = -1);
                        }
                    }
                }
                ++i;
            }
            else {
                break;
            }
        }
    }
    
    private static void maybeAddLeadingDot(char[] array, int[] array2) {
        if (array[0] == '\0') {
            return;
        }
        int length;
        int n;
        for (length = array2.length, n = 0; n < length && array2[n] < 0; ++n) {}
        if (n >= length || n == 0) {
            return;
        }
        int n2;
        for (n2 = array2[n]; n2 < array.length && array[n2] != ':' && array[n2] != '\0'; ++n2) {}
        if (n2 >= array.length || array[n2] == '\0') {
            return;
        }
        array[0] = '.';
        array[1] = '\0';
        array2[0] = 0;
        return;
    }
    
    private static String normalize(String anObject) {
        int needsNormalization = needsNormalization(anObject);
        if (needsNormalization < 0) {
            return anObject;
        }
        char[] charArray = anObject.toCharArray();
        int[] array = new int[needsNormalization];
        split(charArray, array);
        removeDots(charArray, array);
        maybeAddLeadingDot(charArray, array);
        String s = new String(charArray, 0, join(charArray, array));
        if (s.equals(anObject)) {
            return anObject;
        }
        return s;
    }
    
    private static long lowMask(String s) {
        int length = s.length();
        long n = 0L;
        for (int i = 0; i < length; ++i) {
            int char1 = s.charAt(i);
            if (char1 < '@') {
                n |= 1L << char1;
            }
        }
        return n;
    }
    
    private static long highMask(String s) {
        int length = s.length();
        long n = 0L;
        for (int i = 0; i < length; ++i) {
            int char1 = s.charAt(i);
            if (char1 >= '@' && char1 < '\u0080') {
                n |= 1L << char1 - 64;
            }
        }
        return n;
    }
    
    private static long lowMask(char a, char a2) {
        long n = 0L;
        int i;
        int max;
        for (i = Math.max(Math.min((int)a, 63), 0), max = Math.max(Math.min((int)a2, 63), 0), ; i <= max; ++i) {
            n |= 1L << i;
        }
        return n;
    }
    
    private static long highMask(char a, char a2) {
        long n = 0L;
        int i;
        int n2;
        for (i = Math.max(Math.min((int)a, 127), 64) - 64, n2 = Math.max(Math.min((int)a2, 127), 64) - 64, ; i <= n2; ++i) {
            n |= 1L << i;
        }
        return n;
    }
    
    private static boolean match(char c, long n, long n2) {
        if (c == '\0') {
            return false;
        }
        if (c < '@') {
            return (1L << c & n) != 0x0L;
        }
        return c < '\u0080' && (1L << c - 64 & n2) != 0x0L;
    }
    
    private static void appendEscape(StringBuffer sb, byte b) {
        sb.append('%');
        sb.append(URI.hexDigits[b >> 4 & 0xF]);
        sb.append(URI.hexDigits[b >> 0 & 0xF]);
    }
    
    private static void appendEncoded(StringBuffer sb, char c) {
        ByteBuffer encode = null;
        try {
            encode = ThreadLocalCoders.encoderFor("UTF-8").encode(CharBuffer.wrap("" + c));
        }
        catch (CharacterCodingException ex) {
            assert false;
        }
        while (encode.hasRemaining()) {
            int n = encode.get() & 0xFF;
            if (n >= 128) {
                appendEscape(sb, (byte)n);
            }
            else {
                sb.append((char)n);
            }
        }
    }
    
    private static String quote(String s, long n, long n2) {
        s.length();
        Object o = null;
        int n3 = (n & 0x1L) != 0x0L;
        for (int i = 0; i < s.length(); ++i) {
            int char1 = s.charAt(i);
            if (char1 < '\u0080') {
                if (!match((char)char1, n, n2)) {
                    if (o == null) {
                        o = new StringBuffer();
                        o.append(s.substring(0, i));
                    }
                    appendEscape(o, (byte)char1);
                }
                else if (o != null) {
                    o.append((char)char1);
                }
            }
            else if (n3 && (Character.isSpaceChar((char)char1) || Character.isISOControl((char)char1))) {
                if (o == null) {
                    o = new StringBuffer();
                    o.append(s.substring(0, i));
                }
                appendEncoded(o, (char)char1);
            }
            else if (o != null) {
                o.append((char)char1);
            }
        }
        return (o == null) ? s : o.toString();
    }
    
    private static String encode(String src) {
        int length = src.length();
        if (length == 0) {
            return src;
        }
        int index = 0;
        while (src.charAt(index) < '\u0080') {
            if (++index < length) {
                continue;
            }
            return src;
        }
        String normalize = Normalizer.normalize(src, Normalizer.Form.NFC);
        ByteBuffer encode = null;
        try {
            encode = ThreadLocalCoders.encoderFor("UTF-8").encode(CharBuffer.wrap(normalize));
        }
        catch (CharacterCodingException ex) {
            assert false;
        }
        StringBuffer o = new StringBuffer();
        while (encode.hasRemaining()) {
            int n = encode.get() & 0xFF;
            if (n >= 128) {
                appendEscape(o, (byte)n);
            }
            else {
                o.append((char)n);
            }
        }
        return o.toString();
    }
    
    private static int decode(char c) {
        if (c >= '0' && c <= '9') {
            return c - 48;
        }
        if (c >= 'a' && c <= 'f') {
            return c - 97 + 10;
        }
        if (c >= 'A' && c <= 'F') {
            return c - 65 + 10;
        }
        assert false;
        return -1;
    }
    
    private static byte decode(char c, char c2) {
        return (byte)((decode(c) & 0xF) << 4 | (decode(c2) & 0xF) << 0);
    }
    
    private static String decode(String s) {
        if (s == null) {
            return s;
        }
        int length = s.length();
        if (length == 0) {
            return s;
        }
        if (s.indexOf(37) < 0) {
            return s;
        }
        StringBuffer sb = new StringBuffer(length);
        ByteBuffer allocate = ByteBuffer.allocate(length);
        CharBuffer allocate2 = CharBuffer.allocate(length);
        CharsetDecoder onUnmappableCharacter = ThreadLocalCoders.decoderFor("UTF-8").onMalformedInput(CodingErrorAction.REPLACE).onUnmappableCharacter(CodingErrorAction.REPLACE);
        int c = s.charAt(0);
        int n = 0;
        int i = 0;
    Label_0080:
        while (i < length) {
            assert c == s.charAt(i);
            if (c == '[') {
                n = 1;
            }
            else if (n != 0 && c == ']') {
                n = 0;
            }
            if (c != '%' || n != 0) {
                sb.append((char)c);
                if (++i >= length) {
                    break;
                }
                c = s.charAt(i);
            }
            else {
                allocate.clear();
                while (URI.$assertionsDisabled || length - i >= 2) {
                    allocate.put(decode(s.charAt(++i), s.charAt(++i)));
                    if (++i < length) {
                        c = s.charAt(i);
                        if (c == '%') {
                            continue;
                        }
                    }
                    allocate.flip();
                    allocate2.clear();
                    onUnmappableCharacter.reset();
                    CoderResult decode = onUnmappableCharacter.decode(allocate, allocate2, true);
                    assert !(!decode.isUnderflow());
                    CoderResult flush = onUnmappableCharacter.flush(allocate2);
                    assert !(!flush.isUnderflow());
                    sb.append(allocate2.flip().toString());
                    continue Label_0080;
                }
                throw new AssertionError();
            }
        }
        return sb.toString();
    }
    
    static boolean access$300(char c, long n, long n2) {
        return match(c, n, n2);
    }
    
    static {
        L_DIGIT = lowMask('0', '9');
        H_UPALPHA = highMask('A', 'Z');
        H_LOWALPHA = highMask('a', 'z');
        H_ALPHA = (URI.H_LOWALPHA | URI.H_UPALPHA);
        L_ALPHANUM = (URI.L_DIGIT | 0x0L);
        H_ALPHANUM = (0x0L | URI.H_ALPHA);
        L_HEX = URI.L_DIGIT;
        H_HEX = (highMask('A', 'F') | highMask('a', 'f'));
        L_MARK = lowMask("-_.!~*'()");
        H_MARK = highMask("-_.!~*'()");
        L_UNRESERVED = (URI.L_ALPHANUM | URI.L_MARK);
        H_UNRESERVED = (URI.H_ALPHANUM | URI.H_MARK);
        L_RESERVED = lowMask(";/?:@&=+$,[]");
        H_RESERVED = highMask(";/?:@&=+$,[]");
        L_URIC = (URI.L_RESERVED | URI.L_UNRESERVED | 0x1L);
        H_URIC = (URI.H_RESERVED | URI.H_UNRESERVED | 0x0L);
        L_PCHAR = (URI.L_UNRESERVED | 0x1L | lowMask(":@&=+$,"));
        H_PCHAR = (URI.H_UNRESERVED | 0x0L | highMask(":@&=+$,"));
        L_PATH = (URI.L_PCHAR | lowMask(";/"));
        H_PATH = (URI.H_PCHAR | highMask(";/"));
        L_DASH = lowMask("-");
        H_DASH = highMask("-");
        L_DOT = lowMask(".");
        H_DOT = highMask(".");
        L_USERINFO = (URI.L_UNRESERVED | 0x1L | lowMask(";:&=+$,"));
        H_USERINFO = (URI.H_UNRESERVED | 0x0L | highMask(";:&=+$,"));
        L_REG_NAME = (URI.L_UNRESERVED | 0x1L | lowMask("$,;:@&=+"));
        H_REG_NAME = (URI.H_UNRESERVED | 0x0L | highMask("$,;:@&=+"));
        L_SERVER = (URI.L_USERINFO | URI.L_ALPHANUM | URI.L_DASH | lowMask(".:@[]"));
        H_SERVER = (URI.H_USERINFO | URI.H_ALPHANUM | URI.H_DASH | highMask(".:@[]"));
        L_SERVER_PERCENT = (URI.L_SERVER | lowMask("%"));
        H_SERVER_PERCENT = (URI.H_SERVER | highMask("%"));
        L_LEFT_BRACKET = lowMask("[");
        H_LEFT_BRACKET = highMask("[");
        L_SCHEME = (0x0L | URI.L_DIGIT | lowMask("+-."));
        H_SCHEME = (URI.H_ALPHA | 0x0L | highMask("+-."));
        L_URIC_NO_SLASH = (URI.L_UNRESERVED | 0x1L | lowMask(";?:@&=+$,"));
        H_URIC_NO_SLASH = (URI.H_UNRESERVED | 0x0L | highMask(";?:@&=+$,"));
        hexDigits = new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
    }
    
    private class Parser
    {
        private String input;
        private boolean requireServerAuthority;
        private int ipv6byteCount;
        
        Parser(String input) {
            this.requireServerAuthority = false;
            this.ipv6byteCount = 0;
            URI.this.string = (this.input = input);
        }
        
        private void fail(String s) throws URISyntaxException {
            throw new URISyntaxException(this.input, s);
        }
        
        private void fail(String s, int n) throws URISyntaxException {
            throw new URISyntaxException(this.input, s, n);
        }
        
        private void failExpecting(String str, int n) throws URISyntaxException {
            this.fail("Expected " + str, n);
        }
        
        private void failExpecting(String str, String str2, int n) throws URISyntaxException {
            this.fail("Expected " + str + " following " + str2, n);
        }
        
        private String substring(int beginIndex, int endIndex) {
            return this.input.substring(beginIndex, endIndex);
        }
        
        private char charAt(int index) {
            return this.input.charAt(index);
        }
        
        private boolean at(int n, int n2, char c) {
            return n < n2 && this.charAt(n) == c;
        }
        
        private boolean at(int n, int n2, String s) {
            int length = s.length();
            if (length > n2 - n) {
                return false;
            }
            int index;
            for (index = 0; index < length && this.charAt(n++) == s.charAt(index); ++index) {}
            return index == length;
        }
        
        private int scan(int n, int n2, char c) {
            if (n < n2 && this.charAt(n) == c) {
                return n + 1;
            }
            return n;
        }
        
        private int scan(int i, int n, String s, String s2) {
            while (i < n) {
                int char1 = this.charAt(i);
                if (s.indexOf(char1) >= 0) {
                    return -1;
                }
                if (s2.indexOf(char1) >= 0) {
                    break;
                }
                ++i;
            }
            return i;
        }
        
        private int scanEscape(int n, int n2, char c) throws URISyntaxException {
            if (c == '%') {
                if (n + 3 <= n2 && URI.access$300(this.charAt(n + 1), URI.L_HEX, URI.H_HEX) && URI.access$300(this.charAt(n + 2), URI.L_HEX, URI.H_HEX)) {
                    return n + 3;
                }
                this.fail("Malformed escape pair", n);
            }
            else if (c > '\u0080' && !Character.isSpaceChar(c) && !Character.isISOControl(c)) {
                return n + 1;
            }
            return n;
        }
        
        private int scan(int i, int n, long n2, long n3) throws URISyntaxException {
            while (i < n) {
                int char1 = this.charAt(i);
                if (URI.access$300((char)char1, n2, n3)) {
                    ++i;
                }
                else if ((n2 & 0x1L) != 0x0L) {
                    int scanEscape = this.scanEscape(i, n, (char)char1);
                    if (scanEscape > i) {
                        i = scanEscape;
                    }
                }
                else {
                    break;
                }
            }
            return i;
        }
        
        private void checkChars(int n, int n2, long n3, long n4, String str) throws URISyntaxException {
            int scan = this.scan(n, n2, n3, n4);
            if (scan < n2) {
                this.fail("Illegal character in " + str, scan);
            }
        }
        
        private void checkChar(int n, long n2, long n3, String s) throws URISyntaxException {
            this.checkChars(n, n + 1, n2, n3, s);
        }
        
        void parse(boolean requireServerAuthority) throws URISyntaxException {
            this.requireServerAuthority = requireServerAuthority;
            int n = this.input.length();
            int scan = this.scan(0, n, "/?#", ":");
            if (scan >= 0 && this.at(scan, n, ':')) {
                if (scan == 0) {
                    this.failExpecting("scheme name", 0);
                }
                this.checkChar(0, 0L, URI.H_ALPHA, "scheme name");
                this.checkChars(1, scan, URI.L_SCHEME, URI.H_SCHEME, "scheme name");
                URI.this.scheme = this.substring(0, scan);
                scan = ++scan;
                if (this.at(scan, n, '/')) {
                    n = this.parseHierarchical(scan, n);
                }
                else {
                    int scan2 = this.scan(scan, n, "", "#");
                    if (scan2 <= scan) {
                        this.failExpecting("scheme-specific part", scan);
                    }
                    this.checkChars(scan, scan2, URI.L_URIC, URI.H_URIC, "opaque part");
                    n = scan2;
                }
            }
            else {
                scan = 0;
                n = this.parseHierarchical(0, n);
            }
            URI.this.schemeSpecificPart = this.substring(scan, n);
            if (this.at(n, n, '#')) {
                this.checkChars(n + 1, n, URI.L_URIC, URI.H_URIC, "fragment");
                URI.this.fragment = this.substring(n + 1, n);
            }
            if (n < n) {
                this.fail("end of URI", n);
            }
        }
        
        private int parseHierarchical(int authority, int n) throws URISyntaxException {
            if (this.at(authority, n, '/') && this.at(authority + 1, n, '/')) {
                authority += 2;
                int scan = this.scan(authority, n, "", "/?#");
                if (scan > authority) {
                    authority = this.parseAuthority(authority, scan);
                }
                else if (scan >= n) {
                    this.failExpecting("authority", authority);
                }
            }
            int scan2 = this.scan(authority, n, "", "?#");
            this.checkChars(authority, scan2, URI.L_PATH, URI.H_PATH, "path");
            URI.this.path = this.substring(authority, scan2);
            int n2 = scan2;
            if (this.at(n2, n, '?')) {
                ++n2;
                int scan3 = this.scan(n2, n, "", "#");
                this.checkChars(n2, scan3, URI.L_URIC, URI.H_URIC, "query");
                URI.this.query = this.substring(n2, scan3);
                n2 = scan3;
            }
            return n2;
        }
        
        private int parseAuthority(int server, int n) throws URISyntaxException {
            server = (server = server);
            URISyntaxException ex = null;
            int n2;
            if (this.scan(server, n, "", "]") > server) {
                n2 = (this.scan(server, n, URI.L_SERVER_PERCENT, URI.H_SERVER_PERCENT) == n);
            }
            else {
                n2 = (this.scan(server, n, URI.L_SERVER, URI.H_SERVER) == n);
            }
            int n3 = this.scan(server, n, URI.L_REG_NAME, URI.H_REG_NAME) == n;
            if (n3 && n2 == 0) {
                URI.this.authority = this.substring(server, n);
                return n;
            }
            if (n2 != 0) {
                try {
                    server = this.parseServer(server, n);
                    if (server < n) {
                        this.failExpecting("end of authority", server);
                    }
                    URI.this.authority = this.substring(server, n);
                }
                catch (URISyntaxException ex2) {
                    URI.this.userInfo = null;
                    URI.this.host = null;
                    URI.this.port = -1;
                    if (this.requireServerAuthority) {
                        throw ex2;
                    }
                    ex = ex2;
                }
            }
            if (server < n) {
                if (!(n3 == 0)) {
                    URI.this.authority = this.substring(server, n);
                }
                else {
                    if (ex != null) {
                        throw ex;
                    }
                    this.fail("Illegal character in authority", server);
                }
            }
            return n;
        }
        
        private int parseServer(int n, int n2) throws URISyntaxException {
            int n3 = n;
            int scan = this.scan(n3, n2, "/?#", "@");
            if (scan >= n3 && this.at(scan, n2, '@')) {
                this.checkChars(n3, scan, URI.L_USERINFO, URI.H_USERINFO, "user info");
                URI.this.userInfo = this.substring(n3, scan);
                n3 = scan + 1;
            }
            if (this.at(n3, n2, '[')) {
                ++n3;
                int scan2 = this.scan(n3, n2, "/?#", "]");
                if (scan2 > n3 && this.at(scan2, n2, ']')) {
                    int scan3 = this.scan(n3, scan2, "", "%");
                    if (scan3 > n3) {
                        this.parseIPv6Reference(n3, scan3);
                        if (scan3 + 1 == scan2) {
                            this.fail("scope id expected");
                        }
                        this.checkChars(scan3 + 1, scan2, URI.L_ALPHANUM, URI.H_ALPHANUM, "scope id");
                    }
                    else {
                        this.parseIPv6Reference(n3, scan2);
                    }
                    URI.this.host = this.substring(n3 - 1, scan2 + 1);
                    n3 = scan2 + 1;
                }
                else {
                    this.failExpecting("closing bracket for IPv6 address", scan2);
                }
            }
            else {
                int n4 = this.parseIPv4Address(n3, n2);
                if (n4 <= n3) {
                    n4 = this.parseHostname(n3, n2);
                }
                n3 = n4;
            }
            if (this.at(n3, n2, ':')) {
                ++n3;
                int scan4 = this.scan(n3, n2, "", "/");
                if (scan4 > n3) {
                    this.checkChars(n3, scan4, URI.L_DIGIT, 0L, "port number");
                    try {
                        URI.this.port = Integer.parseInt(this.substring(n3, scan4));
                    }
                    catch (NumberFormatException ex) {
                        this.fail("Malformed port number", n3);
                    }
                    n3 = scan4;
                }
            }
            if (n3 < n2) {
                this.failExpecting("port number", n3);
            }
            return n3;
        }
        
        private int scanByte(int n, int n2) throws URISyntaxException {
            int scan = this.scan(n, n2, URI.L_DIGIT, 0L);
            if (scan <= n) {
                return scan;
            }
            if (Integer.parseInt(this.substring(n, scan)) > 255) {
                return n;
            }
            return scan;
        }
        
        private int scanIPv4Address(int n, int n2, boolean b) throws URISyntaxException {
            int scan = this.scan(n, n2, URI.L_DIGIT | URI.L_DOT, 0x0L | URI.H_DOT);
            if (scan <= n || (b && scan != n2)) {
                return -1;
            }
            int n3;
            if ((n3 = this.scanByte(n, scan)) > n) {
                int n4 = n3;
                if ((n3 = this.scan(n4, scan, '.')) > n4) {
                    int n5 = n3;
                    if ((n3 = this.scanByte(n5, scan)) > n5) {
                        int n6 = n3;
                        if ((n3 = this.scan(n6, scan, '.')) > n6) {
                            int n7 = n3;
                            if ((n3 = this.scanByte(n7, scan)) > n7) {
                                int n8 = n3;
                                if ((n3 = this.scan(n8, scan, '.')) > n8) {
                                    int n9 = n3;
                                    if ((n3 = this.scanByte(n9, scan)) > n9 && n3 >= scan) {
                                        return n3;
                                    }
                                }
                            }
                        }
                    }
                }
            }
            this.fail("Malformed IPv4 address", n3);
            return -1;
        }
        
        private int takeIPv4Address(int n, int n2, String s) throws URISyntaxException {
            int scanIPv4Address = this.scanIPv4Address(n, n2, true);
            if (scanIPv4Address <= n) {
                this.failExpecting(s, n);
            }
            return scanIPv4Address;
        }
        
        private int parseIPv4Address(int n, int n2) {
            int scanIPv4Address;
            try {
                scanIPv4Address = this.scanIPv4Address(n, n2, false);
            }
            catch (URISyntaxException ex) {
                return -1;
            }
            catch (NumberFormatException ex2) {
                return -1;
            }
            if (scanIPv4Address > n && scanIPv4Address < n2 && this.charAt(scanIPv4Address) != ':') {
                scanIPv4Address = -1;
            }
            if (scanIPv4Address > n) {
                URI.this.host = this.substring(n, scanIPv4Address);
            }
            return scanIPv4Address;
        }
        
        private int parseHostname(int n, int n2) throws URISyntaxException {
            int i = n;
            int n3 = -1;
            do {
                int scan = this.scan(i, n2, URI.L_ALPHANUM, URI.H_ALPHANUM);
                if (scan <= i) {
                    break;
                }
                if (scan > (n3 = i)) {
                    i = scan;
                    int scan2 = this.scan(i, n2, URI.L_ALPHANUM | URI.L_DASH, URI.H_ALPHANUM | URI.H_DASH);
                    if (scan2 > i) {
                        if (this.charAt(scan2 - 1) == '-') {
                            this.fail("Illegal character in hostname", scan2 - 1);
                        }
                        i = scan2;
                    }
                }
                int scan3 = this.scan(i, n2, '.');
                if (scan3 <= i) {
                    break;
                }
                i = scan3;
            } while (i < n2);
            if (i < n2 && !this.at(i, n2, ':')) {
                this.fail("Illegal character in hostname", i);
            }
            if (n3 < 0) {
                this.failExpecting("hostname", n);
            }
            if (n3 > n && !URI.access$300(this.charAt(n3), 0L, URI.H_ALPHA)) {
                this.fail("Illegal character in hostname", n3);
            }
            URI.this.host = this.substring(n, i);
            return i;
        }
        
        private int parseIPv6Reference(int n, int n2) throws URISyntaxException {
            boolean n3 = false;
            int scanHexSeq = this.scanHexSeq(n, n2);
            if (scanHexSeq > n) {
                n = scanHexSeq;
                if (this.at(n, n2, "::")) {
                    n3 = true;
                    n = this.scanHexPost(n + 2, n2);
                }
                else if (this.at(n, n2, ':')) {
                    n = this.takeIPv4Address(n + 1, n2, "IPv4 address");
                    this.ipv6byteCount += 4;
                }
            }
            else if (this.at(n, n2, "::")) {
                n3 = true;
                n = this.scanHexPost(n + 2, n2);
            }
            if (n < n2) {
                this.fail("Malformed IPv6 address", n);
            }
            if (this.ipv6byteCount > 16) {
                this.fail("IPv6 address too long", n);
            }
            if (n3 == 0 && this.ipv6byteCount < 16) {
                this.fail("IPv6 address too short", n);
            }
            if (n3 && this.ipv6byteCount == 16) {
                this.fail("Malformed IPv6 address", n);
            }
            return n;
        }
        
        private int scanHexPost(int n, int n2) throws URISyntaxException {
            if (n == n2) {
                return n;
            }
            int scanHexSeq = this.scanHexSeq(n, n2);
            int n3;
            if (scanHexSeq > n) {
                n3 = scanHexSeq;
                if (this.at(n3, n2, ':')) {
                    ++n3;
                    n3 = this.takeIPv4Address(n3, n2, "hex digits or IPv4 address");
                    this.ipv6byteCount += 4;
                }
            }
            else {
                n3 = this.takeIPv4Address(n, n2, "hex digits or IPv4 address");
                this.ipv6byteCount += 4;
            }
            return n3;
        }
        
        private int scanHexSeq(int n, int n2) throws URISyntaxException {
            int scan = this.scan(n, n2, URI.L_HEX, URI.H_HEX);
            if (scan <= n) {
                return -1;
            }
            if (this.at(scan, n2, '.')) {
                return -1;
            }
            if (scan > n + 4) {
                this.fail("IPv6 hexadecimal digit sequence too long", n);
            }
            this.ipv6byteCount += 2;
            int i = scan;
            while (i < n2) {
                if (!this.at(i, n2, ':')) {
                    break;
                }
                if (this.at(i + 1, n2, ':')) {
                    break;
                }
                ++i;
                int scan2 = this.scan(i, n2, URI.L_HEX, URI.H_HEX);
                if (scan2 <= i) {
                    this.failExpecting("digits for an IPv6 address", i);
                }
                if (this.at(scan2, n2, '.')) {
                    --i;
                    break;
                }
                if (scan2 > i + 4) {
                    this.fail("IPv6 hexadecimal digit sequence too long", i);
                }
                this.ipv6byteCount += 2;
                i = scan2;
            }
            return i;
        }
    }
}
